/*! \file Commons.h
 *
 * \brief C++ porting of common data structures.
 *
 * Many functions of the original FORTRAN code share complex data blocks in
 * form of COMMON blocks. This poses the limit of freezing the structure of
 * the data blocks in the code, therefore implying the necessity to modify
 * the code to adapt it to the input and to recompile before running. C++,
 * on the contrary, offers the possibility to represent the necessary data
 * structures as classes that can dinamically instantiate the shared information
 * in the most convenient format for the current configuration. This approach
 * adds an abstraction layer that lifts the need to modify and recompile the
 * code depending on the input.
 *
 */

#ifndef SRC_INCLUDE_COMMONS_
#define SRC_INCLUDE_COMMONS_

#include <complex>

/*! \brief Representation of the FORTRAN C1 common blocks.
 *
 * C1 common blocks are used to store vector field expansions and geometric
 * properties, such as sphere sizes, positions and cross-sections. These are
 * used by functions such as `aps`, `diel`, `pwma`, `rabas`, `sscr0`, `sscr2`,
 * `wmamp`, `wmasp` and `dme`. QUESTION: correct?
 *
 * Due to the necessity to share the class contents with many functions that
 * need to read and update various fields, all shared members of C1 are declared
 * public (i.e., the class is just a structure with more advanced constructor
 * and destroyer). Further development may go in the direction of creating
 * a better encapsulation, either by unpacking the contents (recommended) or
 * by creating public methods to access protected fields (standard way, but not
 * recommended for HPC applications).
 */
class C1 {
protected:
	//! \brief Number of spheres.
	int nsph;
	//! \brief Maximum order of field expansion.
	int lm;
	//! \brief NLMMT. QUESTION: definition?
	int nlmmt;
public:
	std::complex<double> **rmi;
	std::complex<double> **rei;
	std::complex<double> **w;
	std::complex<double> *fsas;
	std::complex<double> **vints;
	double *sscs;
	double *sexs;
	double *sabs;
	double *sqscs;
	double *sqexs;
	double *sqabs;
	double *gcsv;
	double *rxx;
	double *ryy;
	double *rzz;
	double *ros;
	double **rc;
	int *iog;
	int *nshl;
	std::complex<double> ***sas;

	/*! \brief C1 instance constructor.
	 *
	 * \param ns: `int` Number of spheres.
	 * \param l_max: `int` Maximum order of field expansion.
	 */
	C1(int ns, int l_max);

	//! \brief C1 instance destroyer.
	~C1();
};

/*! \brief Representation of the FORTRAN C2 blocks.
 *
 */
class C2 {
	int nsph, nhspo;
public:
	std::complex<double> *ris;
	std::complex<double> *dlri;
	std::complex<double> *dc0;
	std::complex<double> *vkt;
	double *vsz;

	/*! \brief C2 instance constructor.
	 *
	 * \param ns: `int` Number of spheres.
	 * \param nl: `int`
	 * \param npnt: `int`
	 * \param npntts: `int`
	 */
	C2(int ns, int nl, int npnt, int npntts);

	//! \brief C2 instance destroyer.
	~C2();
};

#endif
